#!./perl -T

BEGIN {
    chdir 't' if -d 't';
    unshift @INC, '../lib';
    print "1..2\n";
}
END {
    print "not ok 1\n" unless $loaded;
}
use File::Glob;
$loaded = 1;
print "ok 1\n";

# all filenames should be tainted
@a = File::Glob::glob("*");
eval { $a = join("",@a), kill 0; 1 };
unless ($@ =~ /Insecure dependency/) {
    print "not ";
}
print "ok 2\n";
